@@ -10,12 +10,12 @@ use crate::{
     connection::{engine::SessionId, AllocSessionError},
     endpoint::LinkFlow,
     link::LinkHandle,
-    session::{AllocLinkError, SessionIncomingItem},
+    session::{error::AllocLinkError, frame::SessionIncomingItem},
 };
 
 #[derive(Debug)]
-pub enum ConnectionControl {
-    Open,
+pub(crate) enum ConnectionControl {
+    // Open,
     Close(Option<definitions::Error>),
     AllocateSession {
         tx: Sender<SessionIncomingItem>,
@@ -27,16 +27,18 @@ pub enum ConnectionControl {
 impl std::fmt::Display for ConnectionControl {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Open => write!(f, "Open"),
+            // Self::Open => write!(f, "Open"),
             Self::Close(err) => write!(f, "Close({:?})", err),
-            Self::AllocateSession {tx:_, responder:_} => write!(f, "AllocateSession"),
-            Self::DeallocateSession(id) => write!(f, "DeallocateSession({})", id)
+            Self::AllocateSession {
+                tx: _,
+                responder: _,
+            } => write!(f, "AllocateSession"),
+            Self::DeallocateSession(id) => write!(f, "DeallocateSession({})", id),
         }
     }
 }
 
-pub enum SessionControl {
-    Begin,
+pub(crate) enum SessionControl {
     End(Option<definitions::Error>),
     AllocateLink {
         link_name: String,
@@ -47,5 +49,3 @@ pub enum SessionControl {
     LinkFlow(LinkFlow),
     Disposition(Disposition),
 }
-
-pub enum LinkControl {}
