@@ -29,10 +29,10 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
     where
         W: Sink<LinkFrame> + Send + Unpin,
     {
-        let handle = self.output_handle.clone().ok_or_else(|| Error::AmqpError {
-            condition: AmqpError::IllegalState,
-            description: Some("Link is not attached".into()),
-        })?;
+        let handle = self
+            .output_handle
+            .clone()
+            .ok_or_else(|| Error::not_attached())?;
 
         let flow = match (link_credit, drain) {
             (Some(link_credit), Some(drain)) => {
@@ -115,10 +115,7 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
         writer
             .send(LinkFrame::Flow(flow))
             .await
-            .map_err(|_| Error::AmqpError {
-                condition: AmqpError::IllegalState,
-                description: Some("Link is not attached".into()),
-            })
+            .map_err(|_| Error::sending_to_session())
     }
 
     async fn on_incomplete_transfer(
@@ -172,13 +169,19 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
         // This only takes care of whether the message is considered
         // sett
         let settled_by_sender = transfer.settled.unwrap_or_else(|| false);
-        let delivery_id = transfer.delivery_id.ok_or_else(|| Error::AmqpError {
-            condition: AmqpError::NotAllowed,
-            description: Some("The delivery-id is not found".into()),
+        let delivery_id = transfer.delivery_id.ok_or_else(|| {
+            Error::Local(definitions::Error::new(
+                AmqpError::NotAllowed,
+                Some("The delivery-id is not found".into()),
+                None,
+            ))
         })?;
-        let delivery_tag = transfer.delivery_tag.ok_or_else(|| Error::AmqpError {
-            condition: AmqpError::NotAllowed,
-            description: Some("The delivery-tag is not found".into()),
+        let delivery_tag = transfer.delivery_tag.ok_or_else(|| {
+            Error::Local(definitions::Error::new(
+                AmqpError::NotAllowed,
+                Some("The delivery-tag is not found".into()),
+                None,
+            ))
         })?;
 
         let (message, delivery_state) = if settled_by_sender {
@@ -194,10 +197,11 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
                 // field to second.
                 if let ReceiverSettleMode::First = &self.rcv_settle_mode {
                     if let ReceiverSettleMode::Second = mode {
-                        return Err(Error::AmqpError {
-                            condition: AmqpError::NotAllowed,
-                            description: Some("Negotiated link value is First".into()),
-                        });
+                        return Err(Error::Local(definitions::Error::new(
+                            AmqpError::NotAllowed,
+                            Some("Negotiated link value is First".into()),
+                            None,
+                        )));
                     }
                 }
                 mode
@@ -260,10 +264,10 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
             }
         };
 
-        let link_output_handle = self.output_handle.clone().ok_or_else(|| Error::AmqpError {
-            condition: AmqpError::IllegalState,
-            description: Some("Link is not attached".into()),
-        })?;
+        let link_output_handle = self
+            .output_handle
+            .clone()
+            .ok_or_else(|| Error::not_attached())?;
 
         let delivery = Delivery {
             link_output_handle,
@@ -320,10 +324,10 @@ impl ReceiverLink for Link<role::Receiver, ReceiverFlowState, DeliveryState> {
             batchable,
         };
         let frame = LinkFrame::Disposition(disposition);
-        writer.send(frame).await.map_err(|_| Error::AmqpError {
-            condition: AmqpError::IllegalState,
-            description: Some("Session is disconnected".into()),
-        })?;
+        writer
+            .send(frame)
+            .await
+            .map_err(|_| Error::sending_to_session())?;
 
         // This is a unit enum, clone should be really cheap
         Ok(())
