@@ -1,3 +1,5 @@
+//! Implements AMQP1.0 Session
+
 use std::{collections::BTreeMap, io};
 
 use async_trait::async_trait;
@@ -7,7 +9,7 @@ use fe2o3_amqp_types::{
         SessionError, TransferNumber,
     },
     performatives::{Attach, Begin, Detach, Disposition, End, Flow, Transfer},
-    primitives::Symbol,
+    primitives::Symbol, states::SessionState,
 };
 use futures_util::{Sink, SinkExt};
 use slab::Slab;
@@ -28,45 +30,22 @@ use crate::{
     Payload,
 };
 
-mod frame;
-pub use frame::*;
-pub mod builder;
-pub mod engine;
-mod error;
-pub use self::error::{AllocLinkError, Error};
+pub(crate) mod engine;
+pub(crate) mod frame;
+pub(crate) mod error;
 
-// 2.5.5 Session States
-// UNMAPPED
-// BEGIN SENT
-// BEGIN RCVD
-// MAPPED END SENT
-// END RCVD
-// DISCARDING
-pub enum SessionState {
-    Unmapped,
+mod builder;
+pub use builder::*;
 
-    BeginSent,
-
-    BeginReceived,
-
-    Mapped,
-
-    EndSent,
-
-    EndReceived,
-
-    Discarding,
-}
-
-/// This is not necessary as these will just stay in the SessionEngine
-/// and not shared with other tasks/threads
-// pub struct SessionFlowState {
-//     next_incoming_id: TransferNumber,
-//     incoming_window: u32,
-//     next_outgoing_id: TransferNumber,
-//     outgoing_window: u32,
-// }
+use self::{
+    error::{AllocLinkError, Error},
+    frame::{SessionFrame, SessionFrameBody},
+};
 
+/// A handle to the [`Session`] event loop
+///
+/// Dropping the handle will also stop the [`Session`] event loop
+#[derive(Debug)]
 pub struct SessionHandle {
     pub(crate) control: mpsc::Sender<SessionControl>,
     engine_handle: JoinHandle<Result<(), Error>>,
@@ -75,12 +54,65 @@ pub struct SessionHandle {
     pub(crate) outgoing: mpsc::Sender<LinkFrame>,
 }
 
+impl Drop for SessionHandle {
+    fn drop(&mut self) {
+        let _ = self.control.try_send(SessionControl::End(None));
+    }
+}
+
 impl SessionHandle {
+    /// Checks if the underlying event loop has stopped
+    pub fn is_ended(&self) -> bool {
+        self.control.is_closed()
+    }
+
     /// End the session
+    ///
+    /// # Panics
+    ///
+    /// Panics if called after any of [`end`](#method.end), [`end_with_error`](#method.end_with_error),
+    /// [`on_end`](#on_end) has beend executed.
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
     pub async fn end(&mut self) -> Result<(), Error> {
         // If sending is unsuccessful, the `SessionEngine` event loop is
         // already dropped, this should be reflected by `JoinError` then.
         let _ = self.control.send(SessionControl::End(None)).await;
+        self.on_end().await
+    }
+
+    /// Alias for [`end`](#method.end)
+    pub async fn close(&mut self) -> Result<(), Error> {
+        self.end().await
+    }
+
+    /// End the session with an error
+    ///
+    /// # Panics
+    ///
+    /// Panics if called after any of [`end`](#method.end), [`end_with_error`](#method.end_with_error),
+    /// [`on_end`](#on_end) has beend executed.    
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
+    pub async fn end_with_error(
+        &mut self,
+        error: impl Into<definitions::Error>,
+    ) -> Result<(), Error> {
+        // If sending is unsuccessful, the `SessionEngine` event loop is
+        // already dropped, this should be reflected by `JoinError` then.
+        let _ = self
+            .control
+            .send(SessionControl::End(Some(error.into())))
+            .await;
+        self.on_end().await
+    }
+
+    /// Returns when the underlying event loop has stopped
+    ///
+    /// # Panics
+    ///
+    /// Panics if called after any of [`end`](#method.end), [`end_with_error`](#method.end_with_error),
+    /// [`on_end`](#on_end) has beend executed.
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
+    pub async fn on_end(&mut self) -> Result<(), Error> {
         match (&mut self.engine_handle).await {
             Ok(res) => res,
             Err(e) => Err(Error::JoinError(e)),
@@ -116,6 +148,41 @@ pub(crate) async fn allocate_link(
     result
 }
 
+/// AMQP1.0 Session
+///
+/// # Begin a new Session with default configuration
+///
+/// ```rust,ignore
+/// use fe2o3_amqp::Session;
+/// 
+/// let session = Session::begin(&mut connection).await.unwrap();
+/// ```
+/// 
+/// ## Default configuration
+/// 
+/// | Field | Default Value |
+/// |-------|---------------|
+/// |`next_outgoing_id`| 0 |
+/// |`incoming_window`| [`DEFAULT_WINDOW`] |
+/// |`outgoing_window`| [`DEFAULT_WINDOW`] |
+/// |`handle_max`| `u32::MAX` |
+/// |`offered_capabilities` | `None` |
+/// |`desired_capabilities`| `None` |
+/// |`Properties`| `None` |
+/// 
+/// # Customize configuration with [`Builder`]
+/// 
+/// The builder should be used if the user would like to customize the configuration
+/// for the session.
+/// 
+/// ```rust, ignore
+/// let session = Session::builder()
+///     .handle_max(128)
+///     .begin(&mut connection)
+///     .await.unwrap();
+/// ```
+/// 
+#[derive(Debug)]
 pub struct Session {
     control: mpsc::Sender<SessionControl>,
     // session_id: usize,
@@ -155,10 +222,32 @@ impl Session {
         Self::begin(conn).await
     }
 
+    /// Creates a builder for [`Session`]
     pub fn builder() -> builder::Builder {
         builder::Builder::new()
     }
 
+    /// Begins a new session with the default configurations
+    /// 
+    /// # Default configuration
+    /// 
+    /// | Field | Default Value |
+    /// |-------|---------------|
+    /// |`next_outgoing_id`| 0 |
+    /// |`incoming_window`| [`DEFAULT_WINDOW`] |
+    /// |`outgoing_window`| [`DEFAULT_WINDOW`] |
+    /// |`handle_max`| `u32::MAX` |
+    /// |`offered_capabilities` | `None` |
+    /// |`desired_capabilities`| `None` |
+    /// |`Properties`| `None` |
+    /// 
+    /// # Example
+    /// 
+    /// ```rust,ignore
+    /// use fe2o3_amqp::Session;
+    /// 
+    /// let session = Session::begin(&mut connection).await.unwrap();
+    /// ```
     pub async fn begin(conn: &mut ConnectionHandle) -> Result<SessionHandle, Error> {
         Session::builder().begin(conn).await
     }
@@ -179,6 +268,10 @@ impl endpoint::Session for Session {
         &mut self.local_state
     }
 
+    fn outgoing_channel(&self) -> u16 {
+        self.outgoing_channel
+    }
+
     fn allocate_link(
         &mut self,
         link_name: String,
@@ -219,7 +312,7 @@ impl endpoint::Session for Session {
         match self.local_state {
             SessionState::Unmapped => self.local_state = SessionState::BeginReceived,
             SessionState::BeginSent => self.local_state = SessionState::Mapped,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // End session with unattached handle?
         }
 
         self.incoming_channel = Some(channel);
@@ -257,22 +350,22 @@ impl endpoint::Session for Session {
                         Err(_) => {
                             // TODO: how should this error be handled?
                             // End with UnattachedHandle?
-                            return Err(Error::from(SessionError::UnattachedHandle))
+                            return Err(Error::session_error(SessionError::UnattachedHandle, None));
+                            // End session with unattached handle?
                         }
                     }
                 }
-                None => return Err(Error::from(SessionError::UnattachedHandle))
+                None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
             },
-            None => {
-                return Err(Error::from(SessionError::UnattachedHandle))
-            }
+            None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
         }
 
         Ok(())
     }
 
     async fn on_incoming_flow(&mut self, _channel: u16, flow: Flow) -> Result<(), Self::Error> {
-        // TODO: handle session flow control
+        // Handle session flow control
+        // 
         // When the endpoint receives a flow frame from its peer, it MUST update the next-incoming-id
         // directly from the next-outgoing-id of the frame, and it MUST update the remote-outgoing-
         // window directly from the outgoing-window of the frame.
@@ -295,7 +388,7 @@ impl endpoint::Session for Session {
             }
         }
 
-        // TODO: handle link flow control
+        // Handle link flow control
         if let Ok(link_flow) = LinkFlow::try_from(flow) {
             match self.link_by_input_handle.get(&link_flow.handle) {
                 Some(output_handle) => match self.local_links.get_mut(output_handle.0 as usize) {
@@ -310,12 +403,14 @@ impl endpoint::Session for Session {
                                 // Sending control to self. This will only give an error if the receiving
                                 // half is dropped. An error would thus indicate that the event loop
                                 // has stopped, so it could be considered illegal state.
-                                .map_err(|_| AmqpError::IllegalState)?;
+                                //
+                                // If the event loop has stopped, this should not be executed at all
+                                .map_err(|_| Error::amqp_error(AmqpError::IllegalState, None))?;
                         }
                     }
-                    None => return Err(SessionError::UnattachedHandle.into()),
+                    None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
                 },
-                None => return Err(SessionError::UnattachedHandle.into()),
+                None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
             }
         }
 
@@ -338,30 +433,32 @@ impl endpoint::Session for Session {
         match self.link_by_input_handle.get(&transfer.handle) {
             Some(output_handle) => match self.local_links.get_mut(output_handle.0 as usize) {
                 Some(link_handle) => {
-                    let delivery_id =
-                        match link_handle.on_incoming_transfer(transfer, payload).await {
-                            Ok(opt) => opt,
-                            Err(_) => {
-                                // TODO: should the link handle be removed from the session?
-                                return Err(SessionError::UnattachedHandle.into());
-                            }
-                        };
+                    let id_and_tag = match link_handle.on_incoming_transfer(transfer, payload).await
+                    {
+                        Ok(opt) => opt,
+                        Err((closed, error)) => {
+                            return Err(Error::LinkHandleError {
+                                handle: output_handle.clone(),
+                                closed,
+                                error,
+                            })
+                        }
+                    };
 
                     // If the unsettled map needs this
-                    if let Some((delivery_id, delivery_tag)) = delivery_id {
+                    if let Some((delivery_id, delivery_tag)) = id_and_tag {
                         self.delivery_tag_by_id
                             .insert(delivery_id, (output_handle.clone(), delivery_tag));
                     }
                 }
-                None => return Err(SessionError::UnattachedHandle.into()),
+                None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
             },
-            None => return Err(SessionError::UnattachedHandle.into()),
+            None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle?
         };
 
         Ok(())
     }
 
-    /// TODO: sender send settled message in response to receivers disposition
     async fn on_incoming_disposition(
         &mut self,
         _channel: u16,
@@ -443,7 +540,8 @@ impl endpoint::Session for Session {
                             self.control
                                 .send(SessionControl::Disposition(disposition))
                                 .await
-                                .map_err(|_| AmqpError::IllegalState)?
+                                .map_err(|_| Error::amqp_error(AmqpError::IllegalState, None))?
+                            // event loop has stopped
                         }
 
                         prev = echo;
@@ -463,17 +561,19 @@ impl endpoint::Session for Session {
         // Remove the link by input handle
         let output_handle = match self.link_by_input_handle.remove(&detach.handle) {
             Some(handle) => handle,
-            None => return Err(Error::from(SessionError::UnattachedHandle)), // End session with unattached handle?
+            None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle
         };
         match self.local_links.get_mut(output_handle.0 as usize) {
             Some(link) => {
                 // TODO:
                 match link.send(LinkFrame::Detach(detach)).await {
                     Ok(_) => {}
-                    Err(_) => return Err(Error::from(SessionError::UnattachedHandle)), // End session with unattached handle?
+                    Err(_) => {
+                        return Err(Error::session_error(SessionError::UnattachedHandle, None))
+                    } // End session with unattached handle
                 }
             }
-            None => return Err(Error::from(SessionError::UnattachedHandle)), // End session with unattached handle?
+            None => return Err(Error::session_error(SessionError::UnattachedHandle, None)), // End session with unattached handle
         }
 
         Ok(())
@@ -481,9 +581,7 @@ impl endpoint::Session for Session {
 
     async fn on_incoming_end(&mut self, _channel: u16, end: End) -> Result<(), Self::Error> {
         match self.local_state {
-            SessionState::BeginSent 
-            | SessionState::BeginReceived
-            | SessionState::Mapped => {
+            SessionState::BeginSent | SessionState::BeginReceived | SessionState::Mapped => {
                 self.local_state = SessionState::EndReceived;
                 self.control
                     .send(SessionControl::End(None))
@@ -491,19 +589,19 @@ impl endpoint::Session for Session {
                     // The `SendError` occurs when the receiving half is dropped,
                     // indicating that the `SessionEngine::event_loop` has stopped.
                     // and thus should yield an illegal state error
-                    .map_err(|_| AmqpError::IllegalState)?;
+                    .map_err(|_| Error::amqp_error(AmqpError::IllegalState, None))?;
+                // event loop has stopped
             }
-            SessionState::EndSent
-            | SessionState::Discarding  => {
+            SessionState::EndSent | SessionState::Discarding => {
                 self.local_state = SessionState::Unmapped
             }
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // End session with illegal state?
         }
 
         if let Some(error) = end.error {
             // TODO: handle remote error
             tracing::error!(remote_error = ?error);
-            return Err(Error::RemoteError(error))
+            return Err(Error::Remote(error));
         }
 
         Ok(())
@@ -550,7 +648,7 @@ impl endpoint::Session for Session {
                 })?;
                 self.local_state = SessionState::Mapped;
             }
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // End session with illegal state
         }
 
         Ok(())
@@ -661,7 +759,7 @@ impl endpoint::Session for Session {
                 false => self.local_state = SessionState::EndSent,
             },
             SessionState::EndReceived => self.local_state = SessionState::Unmapped,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // End session with illegal state
         }
 
         let frame = SessionFrame::new(self.outgoing_channel, SessionFrameBody::End(End { error }));
